function [d,nPaths,verticesByDist,child,parent] = BF_count_paths(s,A,nNodes)
%
% [d, sigma] = BF_count_paths(s,A)
%
% s = source node index
% A = adjacency matrix for the graph
% nNodes = size(A,1);
% d(i) = distance from s to i.
% nPaths(i) = # of shortest paths from s to i. 
% verticesByDist(i) = i-th closest node to s (includes node i)
% child{i} = child nodes of i in the shortest path DAG rooted at s
% parent{i} = parent nodes of i in the shortest path DAG rooted at s

% CVS version control block - do not edit manually
%  $RCSfile: BF_count_paths.m,v $
%  $Revision: 1.2 $
%  $Date: 2005/04/26 18:33:19 $
%  $Source: /home/dchua/.cvs/CoB/BF_count_paths.m,v $
% End CVS version control block

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Implementation notes:
%
% 1. This implementation uses queues at the innermost loop. From the
% standpoint of clarity this should be implemented via a
% functions/methods. However, matlab only implement "pass by value"
% for function arguments, so for reasons of speed the methods are
% hand-coded in place. (Ugh!)
%
% 2. Conventions for the queues: The queue is empty when the
% idxTail < idxHead 

parent = cell(nNodes, 1);
child = cell(nNodes, 1);
nPaths = zeros(nNodes, 1);
d = NaN(nNodes, 1);
verticesByDist = zeros(nNodes, 1);  % The queue and verticesByDistance
verticesByDist_head = 1; % Should write a function to handle this, instead of 
verticesByDist_tail = 0; % this kludge, but the queue is going to be used a lot

verticesByDist_tail = verticesByDist_tail + 1;
verticesByDist(verticesByDist_tail) = s;
d(s) = 0;
nPaths(s) = 1;

while verticesByDist_head <= verticesByDist_tail % i.e., queue non-empty
  v = verticesByDist(verticesByDist_head);
  verticesByDist_head = verticesByDist_head + 1;
% $$$   disp(['Processing node ' num2str(v)]);
  for u = find(A(v,:))
    if isnan(d(u))      % First time we visit this node
      d(u) = d(v) + 1;  % Set the distance
      nPaths(u) = nPaths(v);
      % Add it to the queue
      verticesByDist_tail = verticesByDist_tail + 1;
      verticesByDist(verticesByDist_tail) = u;
      % And record the tree structure
      child{v} = [child{v}, u];
      parent{u} = [parent{u}, v];
    elseif d(u) == d(v) + 1
      % We've already been here before so increment nPaths(u) and
      % record the additional tree structure.
      nPaths(u) = nPaths(u) + nPaths(v);
      child{v} = [child{v}, u];
      parent{u} = [parent{u}, v];
    end
  end
end
% Trim verticesByDist before returning it. 
verticesByDist = verticesByDist(1:verticesByDist_tail);

%  This code has been developed by David Chua, Eric Kolaczyk, and
%  Mark Barthelemy.  See Kolaczyk, Chua, and Barthelemy (2009),
%  Group-betweenness and cobetweenness: Inter-related notions of coalition
%  centrality.  Social Networks, 31:3, 190-203.

%
%  Copyright (c) David Chua, Eric Kolaczyk,and Marc Barthelemy, 2009.
%  All rights reserved.
