function [] = DF_visit_nodes(s, u, ancestor_nodes, child, score, nPaths)
%
% DF_visit_nodes(s, u, ancestor_nodes, child, score, nPaths)
%
% A recursive function that performs a depth-first traversal of a 
% shortest path DAG, summing contributions to the cobetweenness as
% it goes. 
%
% Function should be initially called as
%
%   DF_visit_nodes(s, s, [], child, score, nPaths)
%
% where 
% s = the root of the shortest path DAG
% child{i} = children of node i in the DAG rooted at s
% score(i) = sum of weights of shortest paths that traverse node i
% nPaths(i) = number of shortest paths from the root s to node i.
%
% WARNING: Uses the global variable CoB_global.

% CVS version control block - do not edit manually
%  $RCSfile: DF_visit_nodes.m,v $
%  $Author: dchua $
%  $Revision: 1.2 $
%  $Date: 2005/04/28 20:07:21 $
%  $Source: /home/dchua/.cvs/CoB/DF_visit_nodes.m,v $
% End of CVS version control block

global CoB_global;

ancestor_nodes(end+1) = u;

for node = ancestor_nodes
  if node ~= s  % ancestor_nodes(1) should always be s...
    CoB_global(u, node) = CoB_global(u, node) + ...
        (score(u)) / nPaths(u);
  end
end

for node = child{u}
  DF_visit_nodes(s, node, ancestor_nodes, child, score, nPaths);
end

%  This code has been developed by David Chua, Eric Kolaczyk, and
%  Mark Barthelemy.  See Kolaczyk, Chua, and Barthelemy (2009),
%  Group-betweenness and cobetweenness: Inter-related notions of coalition
%  centrality.  Social Networks, 31:3, 190-203.

%
%  Copyright (c) David Chua, Eric Kolaczyk,and Marc Barthelemy, 2009.
%  All rights reserved.
