function [score] = ...
 score_res_nodes(s,verticesByDist,child,parent,nPaths,nNodes,CoalInd)
%
% score = score_nodes(s,verticesByDist,child,parent,nPaths,nNodes)
%
% Uses the output of either BF_count_paths or
% BF_count_paths_weighted to compute the score of each node. 
% This version differs from score_nodes in its original form, in
% that contributions from paths (s,t), for nodes t in CoalInd, are not 
% counted.  (Note: Contributions from nodes s in CoalInd are omitted
% by restricting the range of the summation in 
% compute_CoB_restricted_nodes).
%
% score(i) = the score of node i, this is the sum of the weights of
%            shortest paths that start from s and traverse node i.
%            Note that the shortest paths terminating at node i are
%            not included. (Add one to score(i) if you need that
%            number.)
%
% s = index of the start node
% verticesByDist = nodes of the graph arranged in non-decreasing
%                  order of their distance from node s.
% child{i} = children of node i in the shortest-path DAG rooted at
%            node s.
% parent{i} = parents of node i in the shortest-path DAG rooted at
%             node s.
% nPaths(i) = number of shortest paths from s to i.
% nNodes = the number of nodes in the graph. 
% CoalInd = vector of coalition nodes to be omitted

% CVS version control block - do not edit manually
%  $RCSfile: score_nodes.m,v $
%  $Author: dchua $
%  $Revision: 1.1 $
%  $Date: 2005/04/17 22:48:41 $
%  $Source: /home/dchua/.cvs/CoB/score_nodes.m,v $
% End of CVS version control block

score = zeros(nNodes,1);

% Pointers into verticesByDist
% $$$ verticesByDist_head = 1;
verticesByDist_tail = length(verticesByDist);

% $$$ while(verticesByDist_head <= verticesByDist_tail)
while(1 <= verticesByDist_tail)
  v = verticesByDist(verticesByDist_tail);
  verticesByDist_tail = verticesByDist_tail - 1;
  if sum((v==CoalInd))==0,
    outflux = 1 + score(v);
  else
    outflux = score(v);
  end

  score(parent{v}) = score(parent{v}) ...
      + outflux * nPaths(parent{v}) / nPaths(v);
  % The above line is equivalent to the following for-loop
  % for u = parent{v}
  %   score(u) = score(u) + outflux * nPaths(u) / nPaths(v);
  % end
  
  % In the following block from score_edges, we only touch an edge once
  % so there is no need to sum up the scores. In the node version we
  % will have to sum the scores.
% $$$   for u = parent{v}
% $$$     score(u,v) = outflux * nPaths(u) / nPaths(v);
% $$$   end
end


%  This code has been developed by David Chua, Eric Kolaczyk, and
%  Mark Barthelemy.  See Chua, Kolaczyk, and Barthelemy (2009),
%  Group-betweenness and cobetweenness: Inter-related notions of coalition
%  centrality.  Social Networks, 31:3, 190-203.

%
%  Copyright (c) David Chua, Eric Kolaczyk,and Marc Barthelemy, 2009.
%  All rights reserved.
